clusterplot1 <- function(hclust.x, hclust.y, X,
                         labels.x=hclust.x$labels,
                         labels.y=hclust.y$labels,
                         dend.x.opt=list(),
                         dend.y.opt=list(),
                         labels.x.opt=list(),
                         labels.y.opt=list(),
                         image.opt=list(),
                         layout.opt=list(),
                         cex=1, spc.x=1/2, spc.y=1/2, indent=3/4) {
  
  old.par <- par(no.readonly=TRUE)
  on.exit(par(old.par))
  require(mva)

  if (length(hclust.x$order) != ncol(X) ||
      length(hclust.y$order) != nrow(X)) {
    stop(paste('dim(X) must match number of leaves on the correpsonding',
               'dendrogram.\nTranspose?'))
  }
  if (is.null(layout.opt$widths)) layout.opt$widths <- c(.2, .8)
  if (is.null(layout.opt$heights)) layout.opt$heights <- c(.2, .8)
  
  do.call("layout", c(list(mat=matrix(c(0, 2, 3, 1), 2),
                           respect=FALSE), layout.opt))
  frame()
  par(cex=cex, new=TRUE, bty='n', xaxt='n', yaxt='n', ann=FALSE)

  labs.x <- as.matrix(as.matrix(labels.x)[hclust.x$order, ])
  labs.y <- as.matrix(as.matrix(labels.y)[hclust.y$order, ])
  nc.x <- ncol(labs.x);  nr.x <- nrow(labs.x)
  nc.y <- ncol(labs.y);  nr.y <- nrow(labs.y)
  
  inch2line <- function(x) {
    ### lpi <- (par('mar') / par('mai'))[1]
    ### x * lpi
    x * 2032 / 375 * 1 / cex
  }
  
  ## get lines for mtext() & adjust mar for image()
  widths.y <- inch2line(apply(labs.y, 2, function(xx)
                              max(strwidth(xx, units='inches',
                                           cex=par('cex')))))
  widths.x <- inch2line(apply(labs.x, 2, function(xx)
                              max(strwidth(xx, units='inches',
                                           cex=par('cex')))))
  widths.x <- rev(widths.x)
  if (nc.y > 1) 
    widths.y[1:(nc.y-1)] <- widths.y[1:(nc.y-1)] + spc.y
  if (nc.x > 1)
    widths.x[1:(nc.x-1)] <- widths.x[1:(nc.x-1)] + spc.x
  
  xpos <- cumsum(widths.y) + indent
  ypos <- cumsum(widths.x) + indent
  mar4 <- max(xpos)
  mar1 <- max(ypos)
  xpos <- c(indent, xpos)[1:nc.y]
  ypos <- c(indent, ypos)[1:nc.x]

  tmp.mar <- par('mar')
  par(mar=c(tmp.mar[1]+mar1, 0, 0, tmp.mar[4]+mar4))

  tmp.X <- t(as.matrix(X[hclust.y$order, hclust.x$order]))
  if (!is.null(image.opt$zlim)) {
    tmp.X[tmp.X < min(image.opt$zlim)] <- min(image.opt$zlim)
    tmp.X[tmp.X > max(image.opt$zlim)] <- max(image.opt$zlim)
  }
  do.call('image.default', c(list(x=tmp.X), image.opt))
  box(which='plot', bty='o')
  
  ## since 'midpoints' of image are on [0, 1]
  at1 <- seq(0, 1, length=nr.y)
  axis(4, at=at1, labels=NA, tick=TRUE, line=0, las=2, cex=cex, yaxt='s')
  at2 <- seq(0, 1, length=nr.x)
  axis(1, at=at2, labels=NA, tick=TRUE, line=0, las=2, cex=cex, xaxt='s')
  
  ## Draw text
  if (is.null(labels.y.opt$col)) {
    cols.y <- matrix(par('col.axis'), nr.y, nc.y)
  } else {
    cols.y <- as.matrix(labels.y.opt$col)[hclust.y$order, ]
    labels.y.opt$col <- NULL
  }
  for(i in 1:nc.y) {
    do.call('mtext', c(list(text=labs.y[, i], side=4, line=xpos[i],
                            at=at1, las=2, cex=cex,
                            col=cols.y[, i]),
                       labels.y.opt))
  }
  
  if (is.null(labels.x.opt$col)) {
    cols.x <- matrix(par('col.axis'), nr.x, nc.x)
  } else {
    cols.x <- as.matrix(labels.x.opt$col)[hclust.x$order, ]
    labels.x.opt$col <- NULL
  }
  for(i in 1:nc.x) {
    do.call('mtext', c(list(text=labs.x[, nc.x+1-i], side=1, line=ypos[i],
                            at=at2, las=2, cex=cex,
                            col=labels.x.opt$col[, i]),
                       labels.x.opt))
  }

  ## draw dendrograms
  par(yaxs='i', xaxs='i')
  par(mar=c(tmp.mar[1]+mar1, tmp.mar[2], 0, 0))
  do.call('plot.dendrogram',
          c(list(x=as.dendrogram(hclust.y), horiz=TRUE), dend.y.opt))
  par(mar=c(0, 0, tmp.mar[3], tmp.mar[4]+mar4))
  do.call('plot.dendrogram',
          c(list(x=as.dendrogram(hclust.x), horiz=FALSE), dend.x.opt))
}


clusterplot2 <- function(hclust, X,
                         labels=hclust$labels,
                         dend.opt=list(),
                         labels.opt=list(),
                         image.opt=list(),
                         layout.opt=list(), cex=1, spc=1/2, indent=3/4) {
  
  old.par <- par(no.readonly=TRUE)
  on.exit(par(old.par))
  require(mva)

  if (length(hclust$order) != nrow(X)) {
    stop(paste('nrow(X) must match number of leaves on the correpsonding',
               'dendrogram.\nTranspose?'))
  }
  if (is.null(layout.opt$widths)) layout.opt$widths <- c(.2, .8)
  
  do.call("layout", c(list(mat=matrix(c(2, 1), 1),
                           respect=FALSE), layout.opt))
  frame()
  par(cex=cex, new=TRUE, bty='n', xaxt='n', yaxt='n', ann=FALSE)

  labs <- as.matrix(as.matrix(labels)[hclust$order, ])
  nc <- ncol(labs);  nr <- nrow(labs)
  
  inch2line <- function(x) {
    ## FIX this function.  Seems bunched up when cex != 1
    ## Currently using empirical values
    ### lpi <- (par('mar') / par('mai'))[1]
    ### x * lpi
    x * 2032 / 375 * 1 / cex
  }

  ## get lines for mtext() & adjust mar for image()
  widths <- inch2line(apply(labs, 2, function(xx)
                            max(strwidth(xx, units='inches', cex=cex))))
  if (nc > 1) 
    widths[1:(nc-1)] <- widths[1:(nc-1)] + spc
  
  xpos <- cumsum(widths) + indent
  mar4 <- max(xpos)
  xpos <- c(indent, xpos)[1:nc]

  tmp.mar <- par('mar')
  par(mar=c(tmp.mar[1], 0, tmp.mar[3], tmp.mar[4]+mar4))

  tmp.X <- t(as.matrix(X[hclust$order, ]))
  if (!is.null(image.opt$zlim)) {
    tmp.X[tmp.X < min(image.opt$zlim)] <- min(image.opt$zlim)
    tmp.X[tmp.X > max(image.opt$zlim)] <- max(image.opt$zlim)
  }
  do.call('image.default', c(list(x=tmp.X), image.opt))
  box(which='plot', bty='o')
  
  ## since 'midpoints' of image are on [0, 1]
  at1 <- seq(0, 1, length=nr)
  axis(4, at=at1, labels=NA, tick=TRUE, line=0, las=2, cex=cex, yaxt='s')
  
  ## Draw text
  if (is.null(labels.opt$col)) {
    cols <- matrix(par('col.axis'), nr, nc)
  } else {
    cols <- as.matrix(labels.opt$col)[hclust$order, ]
    labels.opt$col <- NULL
  }

  for(i in 1:nc) {
    do.call('mtext', c(list(text=labs[, i], side=4, line=xpos[i],
                            at=at1, las=2, cex=cex,
                            col=cols[, i]),
                       labels.opt))
  }
  
  ## draw dendrogram
  par(yaxs='i', xaxs='i')
  par(mar=c(tmp.mar[1], tmp.mar[2], tmp.mar[3], 0))
  do.call('plot.dendrogram',
          c(list(x=as.dendrogram(hclust), horiz=TRUE), dend.opt))
}

clusterplot <- function(hclust.y, hclust.x, X,
                        labels.x=hclust.x$labels,
                        labels.y=hclust.y$labels,
                        dend.x.opt=list(),
                        dend.y.opt=list(),
                        labels.x.opt=list(),
                        labels.y.opt=list(),
                        image.opt=list(),
                        layout.opt=list(),
                        cex=1, spc.x=1/2, spc.y=1/2, indent=3/4) {

  if (missing(hclust.x) && missing(X)) {
    stop(paste("Can't plot second dendrogram without heat map",
               "Include X?"))

  } else if (missing(hclust.x)) {
    if (missing(X)) {
      ## one dendrogram & labels
      stop("not yet implemented")
    } else {
      ## one dendrogram, heat map, & labels
      clusterplot2(hclust=hclust.y, X=X, labels=labels.y,
                   dend.opt=dend.y.opt, labels.opt=labels.y.opt,
                   image.opt=image.opt, layout.opt=layout.opt,
                   cex=cex, spc=spc.y, indent=indent)
    }
  } else {
    ## two dendrograms, heat map, & labels
    clusterplot1(hclust.y=hclust.y, hclust.x=hclust.x, X=X,
                 labels.x=labels.x, labels.y=labels.y,
                 dend.x.opt=dend.x.opt, dend.y.opt=dend.y.opt,
                 labels.x.opt=labels.x.opt, labels.y.opt=labels.y.opt,
                 image.opt=image.opt, layout.opt=layout.opt,
                 cex=cex, spc.x=spc.x, spc.y=spc.y, indent=indent)
  }
}


### data(fake)
### library(mva)
### hc.samples <- hclust(dist(t(fake.exprs)), method="average")
### hc.genes <- hclust(dist(fake.exprs), method="average")
### 
### par(mar=c(1, 1, 2, 1)+.1)
### clusterplot(hc.genes, hc.samples, fake.exprs, labels.y=fake.genes[,c(1,3)],
###             labels.x=fake.pheno$tubeno,
###             image.opt=list(col=greenred.colors(2000), zlim=c(-3, 3)),
###             cex=.7)
### title("Cluster Plot")
### 
### ## just one dendrogram
### clusterplot(hc.genes, X=fake.exprs, labels.y=fake.genes, cex=.7)
### 
### mycolors <- sapply(fake.pheno, function(xx) {
###   if (is.factor(xx)) as.numeric(xx) else rep(1, length(xx))
### })
### clusterplot(hc.samples, X=t(fake.exprs), labels.y=fake.pheno,
###             labels.y.opt=list(col=mycolors),
###             image.opt=list(col=greenred.colors(2000)))
