\name{clusterplot}
\alias{clusterplot}
\alias{clusterplot1}
\alias{clusterplot2}
\title{Plot hierarchical clusterings}
\description{
  Creates a plot combining one or two dendrograms with a heat map
  representation of the data.
}
\usage{
clusterplot(hclust.y, hclust.x, X, labels.x = hclust.x$labels,
    labels.y = hclust.y$labels, dend.x.opt = list(), dend.y.opt = list(),
    labels.x.opt = list(), labels.y.opt = list(), image.opt = list(),
    layout.opt = list(), cex = 1, spc.x = 1/2, spc.y = 1/2, indent = 3/4)
}
\arguments{
  \item{hclust.y}{hclust object for the y-axis (left).}
  \item{hclust.x}{hclust object for the x-axis (top).}
  \item{X}{Matrix or data frame with as many rows and columns as the
	corresponding hclust has objects.}
  \item{labels.x}{Labels for the x axis (columns). Can be a matrix or
	data frame whose columns label the columns of \code{X}.
	NULL or NA causes an error.}
  \item{labels.y}{}
  \item{dend.x.opt}{A list containing named options for the dendrogram
	on the x-axis (top).}
  \item{dend.y.opt}{A list containing named options for the dendrogram
	on the y-axis (left).}
  \item{labels.x.opt}{A list containing named options for the labels on
	the x-axis, such as col.  It's unwise to set cex here, since the
	argument cex determines margin widths for the heat map. See cex
	below.}
  \item{labels.y.opt}{A list of options for the labels on the
	y-axis. See labels.x.opt above.}
  \item{image.opt}{A list containing named options for the heat map,
	such as col and zlim.  zlim works slightly different here from
	image.}
  \item{layout.opt}{A list containing named options for the proportions
	of the device used for each of the dendrograms and heat map, such as
	widths and heights.}
  \item{cex}{Character expansion used for BOTH x- & y-axis labels}
  \item{spc.x}{Column spacing, in lines, for x-axis labels}
  \item{spc.y}{Column spacing, in lines, for y-axis labels}
  \item{indent}{Distance, in lines, from tick marks to labels}
}
\value{
}
\details{
  All of the .x arguments are optional.
}
\seealso{
  \code{\link{mtext}}, which draws the labels; \code{\link{layout}},
  which controls the layout of dendrograms and heat map;
  \code{\link{dendrogram}} in package mva; \code{\link{image}}.
}
\examples{
data(fake)
library(mva)
hc.samples <- hclust(dist(t(fake.exprs)), method="average")
hc.genes <- hclust(dist(fake.exprs), method="average")

par(mar=c(1, 1, 2, 1)+.1)
clusterplot(hc.genes, hc.samples, fake.exprs, labels.y=fake.genes[,c(1,3)],
            labels.x=fake.pheno$tubeno,
            image.opt=list(col=greenred.colors(2000), zlim=c(-3, 3)),
            cex=.7)
title("Cluster Plot")

## just one dendrogram
clusterplot(hc.genes, X=fake.exprs, labels.y=fake.genes, cex=.7)

mycolors <- sapply(fake.pheno, function(xx) {
  if (is.factor(xx)) as.numeric(xx) else rep(1, length(xx))
})
clusterplot(hc.samples, X=t(fake.exprs), labels.y=fake.pheno,
            labels.y.opt=list(col=mycolors),
            image.opt=list(col=greenred.colors(2000)))
}
\author{Mark Wilkinson \email{mark.wilkinson@stjude.org}}
\keyword{hplot}
